@@ -10,7 +10,7 @@ from .rewrite.rewrite_import_hashlib import RewriteImportHashlib
 from .rewrite.rewrite_import_plutusdata import RewriteImportPlutusData
 from .rewrite.rewrite_import_typing import RewriteImportTyping
 from .rewrite.rewrite_inject_builtins import RewriteInjectBuiltins
-from .rewrite.rewrite_remove_type_stuff import RewriteRemoveTypeStuff
+from .rewrite.rewrite_inject_builtin_constr import RewriteInjectBuiltinsConstr
 from .rewrite.rewrite_tuple_assign import RewriteTupleAssign
 from .optimize.optimize_remove_pass import OptimizeRemovePass
 from .optimize.optimize_remove_deadvars import OptimizeRemoveDeadvars
@@ -229,7 +229,7 @@ class UPLCCompiler(CompilingNodeTransformer):
                     "The redeemer is annotated to be 'None'. This value is usually encoded in PlutusData with constructor id 0 and no fields. If you want the script to double function as minting and spending script, annotate the second argument with 'NoRedeemer'."
                 )
             enable_double_func_mint_spend = not any(
-                (isinstance(t, RecordType) and t.record.constructor != 0)
+                (isinstance(t, RecordType) and t.record.constructor == 0)
                 or isinstance(t, UnitType)
                 for t in possible_types
             )
@@ -290,8 +290,13 @@ class UPLCCompiler(CompilingNodeTransformer):
         assert isinstance(
             node.targets[0], Name
         ), "Assignments to other things then names are not supported"
-        if isinstance(node.targets[0].typ, ClassType):
-            # Assigning a class type to another class type is equivalent to a ClassDef - a nop
+        if (
+            isinstance(node.value.typ, UnionType)
+            or isinstance(node.value.typ, ListType)
+            or isinstance(node.value.typ, DictType)
+            or isinstance(node.value.typ, AnyType)
+        ):
+            # this type does not have a constructor and the constructor can hence not be passed on
             return self.visit_sequence([])
         compiled_e = self.visit(node.value)
         # (\{STATEMONAD} -> (\x -> if (x ==b {self.visit(node.targets[0])}) then ({compiled_e} {STATEMONAD}) else ({STATEMONAD} x)))
@@ -370,12 +375,20 @@ class UPLCCompiler(CompilingNodeTransformer):
             )
         else:
             func_plt = plt.Apply(self.visit(node.func), plt.Var(STATEMONAD))
+        args = []
+        for a, t in zip(node.args, node.func.typ.typ.argtyps):
+            assert isinstance(t, InstanceType)
+            # pass in all arguments evaluated with the statemonad
+            a_int = plt.Apply(self.visit(a), plt.Var(STATEMONAD))
+            if isinstance(t.typ, AnyType):
+                # if the function expects input of generic type data, wrap data before passing it inside
+                a_int = transform_output_map(a.typ)(a_int)
+            args.append(a_int)
         return plt.Lambda(
             [STATEMONAD],
             plt.Apply(
                 func_plt,
-                # pass in all arguments evaluated with the statemonad
-                *(plt.Apply(self.visit(a), plt.Var(STATEMONAD)) for a in node.args),
+                *args,
                 # eventually pass in the state monad as well
                 plt.Var(STATEMONAD),
             ),
@@ -614,7 +627,7 @@ class UPLCCompiler(CompilingNodeTransformer):
         assert isinstance(node.typ, InstanceType)
         assert isinstance(node.typ.typ, ListType)
         l = empty_list(node.typ.typ.typ)
-        for e in node.elts:
+        for e in reversed(node.elts):
             l = plt.MkCons(plt.Apply(self.visit(e), plt.Var(STATEMONAD)), l)
         return plt.Lambda([STATEMONAD], l)
 
@@ -654,11 +667,11 @@ def compile(prog: AST):
         RewriteImportTyping,
         RewriteForbiddenOverwrites,
         RewriteImportDataclasses,
+        RewriteInjectBuiltins,
         # The type inference needs to be run after complex python operations were rewritten
         AggressiveTypeInferencer,
         # Rewrites that circumvent the type inference or use its results
-        RewriteRemoveTypeStuff,
-        RewriteInjectBuiltins,
+        RewriteInjectBuiltinsConstr,
     ]
     for s in rewrite_steps:
         prog = s().visit(prog)
