@@ -371,6 +371,89 @@ class InstanceType(Type):
 
 @dataclass(frozen=True, unsafe_hash=True)
 class IntegerType(AtomicType):
+    def constr_type(self) -> InstanceType:
+        return InstanceType(FunctionType([StringInstanceType], InstanceType(self)))
+
+    def constr(self) -> plt.AST:
+        return plt.Lambda(
+            ["x", "_"],
+            plt.Let(
+                [
+                    ("e", plt.EncodeUtf8(plt.Var("x"))),
+                    ("len", plt.LengthOfByteString(plt.Var("e"))),
+                    (
+                        "fold_start",
+                        plt.Lambda(
+                            ["start"],
+                            plt.FoldList(
+                                plt.Range(plt.Var("len"), plt.Var("start")),
+                                plt.Lambda(
+                                    ["s", "i"],
+                                    plt.Let(
+                                        [
+                                            (
+                                                "b",
+                                                plt.IndexByteString(
+                                                    plt.Var("e"), plt.Var("i")
+                                                ),
+                                            )
+                                        ],
+                                        plt.Ite(
+                                            plt.EqualsInteger(
+                                                plt.Var("b"), plt.Integer(ord("_"))
+                                            ),
+                                            plt.Var("s"),
+                                            plt.Ite(
+                                                plt.Or(
+                                                    plt.LessThanInteger(
+                                                        plt.Var("b"),
+                                                        plt.Integer(ord("0")),
+                                                    ),
+                                                    plt.LessThanInteger(
+                                                        plt.Integer(ord("9")),
+                                                        plt.Var("b"),
+                                                    ),
+                                                ),
+                                                plt.TraceError(
+                                                    "ValueError: invalid literal for int() with base 10"
+                                                ),
+                                                plt.AddInteger(
+                                                    plt.SubtractInteger(
+                                                        plt.Var("b"),
+                                                        plt.Integer(ord("0")),
+                                                    ),
+                                                    plt.MultiplyInteger(
+                                                        plt.Var("s"), plt.Integer(10)
+                                                    ),
+                                                ),
+                                            ),
+                                        ),
+                                    ),
+                                ),
+                                plt.Integer(0),
+                            ),
+                        ),
+                    ),
+                ],
+                plt.Ite(
+                    plt.EqualsInteger(plt.Var("len"), plt.Integer(0)),
+                    plt.TraceError(
+                        "ValueError: invalid literal for int() with base 10"
+                    ),
+                    plt.Ite(
+                        plt.EqualsInteger(
+                            plt.IndexByteString(plt.Var("e"), plt.Integer(0)),
+                            plt.Integer(ord("-")),
+                        ),
+                        plt.Negate(
+                            plt.Apply(plt.Var("fold_start"), plt.Integer(1)),
+                        ),
+                        plt.Apply(plt.Var("fold_start"), plt.Integer(0)),
+                    ),
+                ),
+            ),
+        )
+
     def cmp(self, op: cmpop, o: "Type") -> plt.AST:
         """The implementation of comparing this type to type o via operator op. Returns a lambda that expects as first argument the object itself and as second the comparison."""
         if isinstance(o, BoolType):
@@ -447,6 +530,76 @@ class IntegerType(AtomicType):
 
 @dataclass(frozen=True, unsafe_hash=True)
 class StringType(AtomicType):
+    def constr_type(self) -> InstanceType:
+        return InstanceType(FunctionType([IntegerInstanceType], InstanceType(self)))
+
+    def constr(self) -> plt.AST:
+        # constructs a string representation of an integer
+        return plt.Lambda(
+            ["x", "_"],
+            plt.DecodeUtf8(
+                plt.Let(
+                    [
+                        (
+                            "strlist",
+                            plt.RecFun(
+                                plt.Lambda(
+                                    ["f", "i"],
+                                    plt.Ite(
+                                        plt.LessThanEqualsInteger(
+                                            plt.Var("i"), plt.Integer(0)
+                                        ),
+                                        plt.EmptyIntegerList(),
+                                        plt.MkCons(
+                                            plt.AddInteger(
+                                                plt.ModInteger(
+                                                    plt.Var("i"), plt.Integer(10)
+                                                ),
+                                                plt.Integer(ord("0")),
+                                            ),
+                                            plt.Apply(
+                                                plt.Var("f"),
+                                                plt.Var("f"),
+                                                plt.DivideInteger(
+                                                    plt.Var("i"), plt.Integer(10)
+                                                ),
+                                            ),
+                                        ),
+                                    ),
+                                ),
+                            ),
+                        ),
+                        (
+                            "mkstr",
+                            plt.Lambda(
+                                ["i"],
+                                plt.FoldList(
+                                    plt.Apply(plt.Var("strlist"), plt.Var("i")),
+                                    plt.Lambda(
+                                        ["b", "i"],
+                                        plt.ConsByteString(plt.Var("i"), plt.Var("b")),
+                                    ),
+                                    plt.ByteString(b""),
+                                ),
+                            ),
+                        ),
+                    ],
+                    plt.Ite(
+                        plt.EqualsInteger(plt.Var("x"), plt.Integer(0)),
+                        plt.ByteString(b"0"),
+                        plt.Ite(
+                            plt.LessThanInteger(plt.Var("x"), plt.Integer(0)),
+                            plt.ConsByteString(
+                                plt.Integer(ord("-")),
+                                plt.Apply(plt.Var("mkstr"), plt.Negate(plt.Var("x"))),
+                            ),
+                            plt.Apply(plt.Var("mkstr"), plt.Var("x")),
+                        ),
+                    ),
+                )
+            ),
+        )
+
     def cmp(self, op: cmpop, o: "Type") -> plt.AST:
         if isinstance(o, StringType):
             if isinstance(op, Eq):
@@ -463,6 +616,16 @@ class ByteStringType(AtomicType):
             )
         )
 
+    def constr(self) -> plt.AST:
+        return plt.Lambda(
+            ["xs", "_"],
+            plt.RFoldList(
+                plt.Var("xs"),
+                plt.Lambda(["a", "x"], plt.ConsByteString(plt.Var("x"), plt.Var("a"))),
+                plt.ByteString(b""),
+            ),
+        )
+
     def cmp(self, op: cmpop, o: "Type") -> plt.AST:
         if isinstance(o, ByteStringType):
             if isinstance(op, Eq):
@@ -782,7 +945,7 @@ TransformExtParamsMap = {
     ByteStringInstanceType: lambda x: plt.UnBData(x),
     StringInstanceType: lambda x: plt.DecodeUtf8(plt.UnBData(x)),
     UnitInstanceType: lambda x: plt.Apply(plt.Lambda(["_"], plt.Unit())),
-    BoolInstanceType: lambda x: plt.NotEqualsInteger(x, plt.Integer(0)),
+    BoolInstanceType: lambda x: plt.NotEqualsInteger(plt.UnIData(x), plt.Integer(0)),
 }
 
 
